/*#######################################################################################
AVR Small Webserver 

Copyright (C) 2004 Ulrich Radig
          (C) 2006 Ingo Busker http://www.mikrocontroller.com
#######################################################################################*/

#ifndef _MAIN_H
 #define _MAIN_H

#define __STDIO_FDEVOPEN_COMPAT_12         // use old implementation of fdevopen
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
//#include <avr/signal.h>    // obsolete since avr-gcc V3.4.5
#include <avr/eeprom.h>

// --- Select Networkcard -------
#define USE_RTL8019   1
#define USE_3C5x9     0
//-------------------------------

#define ISA_CTRL          

#define SYSCLK	14745600	//Quarz Frequenz in Hz
//#define SYSCLK      11059200L 

//#define SYSCLK	16000000	//Quarz Frequenz in Hz
#define F_CPU  SYSCLK

#include "mmc.h"
#include "ftpclient.h"
#include "sendmail.h"
#include "auth.h"
#include "time.h"
#include "ntp.h"
#include "interrupt4.h"
#include "fat16.h"
#include "rtl8019.h"
#include "3c5x9.h"
#include "tcp.h"
#include "arp.h"
#include "ip.h"
#include "icmp.h"
#include "clock.h"
#include "uart.h"
#include "ftpd.h"
#include "webpage.h"
#include "telnet.h"
#include "analog.h"
#include "webcam/mca25.h"
#include "config.h"

#include <util/delay.h>    // was <avr/delay.h>  - has moved since avr-gcc 3.4.5
#define delay_ms(a) { if(a<15) _delay_ms(a); else for (unsigned int i=0;i<a;i++) _delay_ms(1); }

extern int checksum (unsigned char * ,unsigned int,unsigned long);
extern void httpd (char* buffer,int *bufferlen);

extern struct str_ARP_Table ARP_Table[];
#define ARP_TABLE_ENTRY_ROUTER      0
#define ARP_TABLE_ENTRY_NTPSERVER   1
#define ARP_TABLE_ENTRY_FTPSERVER   2
#define ARP_TABLE_ENTRY_SMTPSERVER  3
#define NUM_ARP_ENTRIES      4

extern unsigned char mymac[];
extern unsigned char MYIP[];

extern File *fp;
extern char Fbuffer[];
extern char packet[];

// geteilter datenbereich fr steueraufgaben
union vars_u {
	unsigned char type_char[32];
	unsigned int  type_int[16];
} vars;
// adresse des WORDs, in dem die mailflags fuer zu sendende mails stehen (geteilter datenbereich)
#define MAILFLAGS_ADDR	15    // letztes int im Datenbereich fr MailTrigger reservieren !

#define VARS_RW_BYTES 32 // lnge des R/W-bereichs

//buffer for webcam
#define CAM_BUFFER_LEN 512
//extern unsigned char webcam_buffer[];

#define DEFAULT_CAM_ENABLE       0
#define DEFAULT_CAM_STORE_ENABLE 0
#define DEFAULT_CAM_SOTRE_TIME   60                // in minutes
#define DEFAULT_FTP_PUSH_ENABLE  0
#define DEFAULT_NTP_ENABLE 1

#define EXT_TRIGGER_ADDR         27                      // byte-index im globalen datenbereich
#define EXT_TRIGGER_ENABLED      0x01                    // trigger aktiv
#define EXT_TRIGGERED            0x02                    // trigger ausgelst
#define EXT_TRIGGER_STATE        0x04                    // high oder low aktiv
#define EXT_TRIGGER_TIME_ADDR    12                     // int-index fr min. auslsezeit 
#define EXT_TRIGGER_PIN_ADDR     11                     // int-index fr trigger-portpin 
#define DEFAULT_EXT_TRIGGER_ENABLE  0
#define DEFAULT_EXT_TRIGGER_TIME    60                  // min. 60 Sekunden
#define DEFAULT_EXT_TRIGGER_STATE   1                   // 0 = lowaktiv  -  1 = highaktiv
#define DEFAULT_EXT_TRIGGER_PORTPIN 0                   // 0=B4, 1=D2 , 2=D3, 4=D4, 8=D5, 16=D6

extern unsigned char MMC_Present;
////some webcam control stuff:
extern unsigned char webcam_busy_forconn;
extern int webcam_buffer_empty;
extern unsigned char webcam_pic_end;
extern unsigned char webcam_tmp;
extern unsigned char CAM_Enable;
extern unsigned char CAM_SD_Store_Enable;
extern unsigned int CAM_SD_Store_Time;
extern unsigned char FTP_Push_Enable;
extern void SaveCamToSD(char *buffer);

//MTU Size setzen (min. 600 / max. 1400)
#define MTU_SIZE  600

#define LED_RED (PIND & (1 <<7))

#if USE_RTL8019 && USE_3C5x9
  #error You can only select ONE networkcard !
#endif

#ifndef enable_external_int
#include <compat/deprecated.h>
#endif
#ifndef enable_external_int
#error No macro 'enable_external_int()' found! - maybe obsolete in this version of avr-gcc???
#endif
#if defined (__AVR_ATmega103__)
	#define Networkcard_INT_Enable()	enable_external_int (1<<INT4)
    #define Networkcard_INT_Disable()	enable_external_int (0<<INT4)
#endif
	
#if defined (__AVR_ATmega128__)
	#define Networkcard_INT_Enable()	enable_external_int (1<<INT4)
	#define Networkcard_INT_Disable()	enable_external_int (0<<INT4)
#endif
	
#if defined (__AVR_ATmega32__)
	#define Networkcard_INT_Enable()	enable_external_int (1<<INT0)
	#define Networkcard_INT_Disable()	enable_external_int (0<<INT0)
#endif	

#if defined (__AVR_ATmega644__)
	#define Networkcard_INT_Enable()	enable_external_int (1)
	#define Networkcard_INT_Disable()	enable_external_int (0)
#endif	

#if defined (__AVR_ATmega32__)
	#define Networkcard_INT_Set()	MCUCR=0x01;
#endif	
	
#if defined (__AVR_ATmega644__)
	#define Networkcard_INT_Set()	EICRA = 0x03; 
#endif	

#if defined (__AVR_ATmega103__)
	#define Networkcard_INT_Set()	EICR = 0x03;
#endif
	
#if defined (__AVR_ATmega128__)
	#define Networkcard_INT_Set()	EICR = 0x03;
#endif

#endif //_MAIN_H






